@echo off
cls
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::                                                                    ::
:: TClogSummary           Analyzes Windows TelliCast Client logfiles  ::
:: Copyright (C) 2011                                 Ernst Lobsiger  ::
::                                                                    ::
:: This program is free software: you can redistribute it and/or      ::
:: modify it under the terms of the GNU General Public License as     ::
:: published by the Free Software Foundation, either version 3 of     ::
:: the License, or (at your option) any later version.                ::
::                                                                    ::
:: This program is distributed in the hope that it will be useful,    ::
:: but WITHOUT ANY WARRANTY; without even the implied warranty        ::
:: of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.            ::
:: See the GNU General Public License for more details.               ::
::                                                                    ::
:: You should have received a copy of the GNU General Public License  ::
:: along with this program. If not, see http://www.gnu.org/licenses.  ::
::                                                                    ::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


:: TClogSummary.cmd (*.txt)                  2011-02-01
:: Ernst Lobsiger, Huehnerhubelstrasse 27, CH-3123 Belp
::
::
:: This script makes daily summaries of TelliCast Client
:: logfiles. You can set the number logfiles treated with
:: variable Nmax. For 'Nmax=1' just recv.log is analyzed.
::
:: The script expects logging set to 'normal' in recv.ini.
:: Nmax should not exceed log_file_number set in recv.ini.
:: log_file (default recv.log) must be set as in recv.ini.
::
:: USAGE: 
:: Interactve use if clicked (or run without parameters)
:: Non interactive with CLI parameters today | yesterday
:: TClogSummary today                make todays summary
:: TClogSummary yesterday        make yesterdays summary
::
:: Run 'TClogSummary yesterday' scheduled after 00:05 UTC
:: This script checks the system date and takes it as UTC
:: Counting 1 back from here it will find yesterdays date
:: If your system runs on local time and daylight savings
:: it's probably save to schedule 03:00 for yesterday UTC
::
:: The most useful internet resource for scripting I am
:: aware of is http://www.ss64.com       Ernst Lobsiger


:: BEGIN OF PARAMETER SECTION >>>>>>>>>>>>>>>>>>>>>>>>>>

:: All parameters are to be set without quotes ('',"")
:: DOUBLECHECK: No leading or trailing blanks allowed

:: The path parameters are either one single point '.'
:: or an absolute path without the final backslash '\'. 
:: Put something like  '.' (where TClogSummary.cmd is)
:: or 'C:\tellicast\logfiles' (absolute) or even with 
:: blanks 'C:\Program Files\TC_stuff' or maybe %temp%.

:: The computername Windows uses (from system environment)
set comp=%COMPUTERNAME%


:: Version number of your TelliCast Client (recv.exe) 
set client=2.12.1


:: Most probably your basic log_file name is 'recv.log'
set log_file=recv_1.log


:: Readable path (priviledges) whith recv.log.? files
:: Many users will probably have the following setup
:: logpath=C:\Program Files\T-Systems\BusinessTV-IP
set logpath=C:\EUMETCast\logs


:: For temporary output of log lines with 'mydate' stamp
:: Must be a writable path where you have *enough* space
:: With 2 HD's, this should not be where the raw data is
set tmppath=%temp%


:: Writable (priviledges) path where output summary goes 
:: Summary will be named automatically including mydate
set outpath=C:\Reports


:: Treat TelliCast 'recv.log.(Nmax-1)' down to 'recv.log'
:: For Nmax=1 (Nmax>0) just 'recv.log' will be processed
:: To get output for dates as far back as possible, set
:: this to the value of log_file_number in your recv.ini
set Nmax=15


:: Set the maximum number of summaries that will be kept
:: Oldest summaries are deleted first. Smax=0 keeps all.
set Smax=0


:: Set this to 'Y' if you want to display the output file
:: Otherwise set it to 'N'  (automatically N if scheduled)
set disp=Y


:: Set this to 'Y' for debugging, to 'N' if everything works
:: Maybe that's a first hint for your local Windows CMD guru
set debu=Y


:: This should normally stay 'Q', at least that's my sheer hope
:: Set sDate only if the script fails to handle your 'date /T'!
:: 0 [*] mm/dd/[yy]yy ; 1 [*] dd/mm/[yy]yy ; 2 [*] [yy]yy/mm/dd
set sDate=Q


::  Style:<-If you change delimiter make it this length->
set delim================================================


:: <<<<<<<<<<<<<<<<<<<<<<<<<<<<  END OF PARAMETER SECTION


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: *DO ONLY TOUCH THE CODE BELOW IF YOU KNOW WHAT YOU DO*
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::


:: Only works until next parameter section change
if exist "%~dp0TClogParameters.cmd" (
call "%~dp0TClogParameters.cmd"
set srced=Some parameters might have been sourced
) else (
set srced=None of the parameters has been sourced)
                   
:: Don't source the version number
set Ver=0.93beta

set called="From COMMANDLINE, must be a guru"
:: Was CMD called with the /K switch? Don't be fooled if user Smith does a Drive:\path\TClogSummary /K /k
for /F "tokens=2" %%G in ("%cmdcmdline:"=%") do if /I [%%G==[/K (set called=STARTED) else (set called=CALLED)
If [%called%==[STARTED echo TClogSummary was STARTED     ... will auto EXIT


:: OK, I wanted to avoid that, but after I tried a german and french Windows 7 and the 'date' prompt was hell, we have to (just) query the *REGISTRY*!
set iDate=0
if /I [%sDate%==[Q for /F "tokens=3 skip=2" %%Q in ('reg query "HKCU\Control Panel\International" /v iDate') do (set iDate=%%Q) else (set iDate=%sDate%)

:: Let's get 'today' and 'yesterday' (see subroutine at end of the script)
call :get_dates %iDate%


:: This prevents problems when called in a batch chain or from any black console:
:: TClogSummary does not need to change directory. Point '.' expanded to TCShome.
if [%tmppath:~0,1%==[. (set tlog="%~dp0tmplog") else (set tlog="%tmppath%\tmplog")
if [%tmppath:~0,1%==[. (set tdat="%~dp0tmpdat") else (set tdat="%tmppath%\tmpdat")
if [%logpath:~0,1%==[. (set log=%~dp0) else (set log=%logpath%\)
if [%outpath:~0,1%==[. (set out=%~dp0) else (set out=%outpath%\)


:: Cut last \ again
set log=%log:~0,-1%
set out=%out:~0,-1%
:: Backup for Smax!
set org=%out%

:: Cannot get to work IF CONDITION (.) ELSE (.) with command blocks
:: There is still "Spaghetti" in Windows 7 (do not want PowerShell)

if /I [%1==[today set mydate=%today%&set disp=N& goto :BAR
if /I [%1==[yesterday set mydate=%yesterday%&set disp=N& goto :BAR

echo %delim%
echo TClogSummary BAS                   Version %ver%
echo.
echo.
echo TClogSummary Copyright (C) 2011  Ernst Lobsiger
echo This program comes with ABSOLUTELY NO WARRANTY.
echo.
echo.
echo Messages from 'mydate' entered will be analyzed. 
echo If you just hit *return* or *mistype* your date
echo here, the script 'should' replace it with today.
echo From a 'date /T' we think this to be %today%
echo From that we think yesterday must be %yesterday%
echo.
set mydate=
set /P mydate=Enter 'mydate' formated YYYY-MM-DD : 
set mydate=%mydate: =%
echo %mydate% > %tdat%
set mydate=
for /F %%G in ('findstr ^20[0-9][0-9]-[01][0-9]-[0-3][0-9] %tdat%') do set mydate=%%G
if [%mydate%==[ set mydate=%today%
echo %delim%

:BAR

:: Just in case ..
echo %mydate% > %tdat%

:: Naming of possible debug file. Goes to writable outpath. Don't change!
if /I [%debu%==[Y (set debug="%out%\TClogDebug.log") else (set debug=NUL)

del /F "%out%\TClogDebug.log" 2> NUL 

: Got enough information
call :start_debug %1 %2

:: As proposed by David Taylor
echo.
echo Step 1/2 Extracting %mydate% lines from log(s)
echo.

:: Count *existing* files and their lines from recv.log.Nmax down to recv.log.1 and recv.log with FIND.
:: Error messages of CMD go to %debug%. Missing files are reported further down once more by FINDSTR.
set /A Nmax-=1
set head=%log%\%log_file%
set tail=%log%\%log_file%
set Nfiles=0
set Nlines=0
(for /L %%N in (%Nmax%,-1,1) do FOR /F "tokens=3 delims=:" %%G in ('find /I /C ":" "%head%.%%N"') do (
set /A Nfiles+=1
set /A Nlines+=%%G
echo %%G Lines in "%head%.%%N")) 2>>%debug%

:: On a running system this file is open R/W by the TelliCast Client!
(for /F "tokens=3 delims=:" %%G in ('find /I /C ":" "%head%"') do (
set /A Nfiles+=1
set /A Nlines+=%%G
echo %%G Lines in "%head%")) 2>>%debug% 

:: This will also go to the last line of the summary output
echo.
echo Counted number of recv.log Files/Lines: %Nfiles%/%Nlines%

:: As proposed by Ernst Lobsiger in memoriam of the old DOS FidoNet mail "tosser"
echo.
echo Tossing all %mydate% lines in temp file . . .

:: Now toss 'mydate' lines into %tlog% (missing files reported by FINDSTR here)
if exist %tlog% del /F %tlog% 2>> %debug%
for /L %%N in (%Nmax%,-1,1) do findstr %mydate% "%tail%.%%N" 2>>%debug% >> %tlog%
findstr %mydate%.*:.*:.*: "%head%" 2>>%debug% >> %tlog%

:: Restore old
set /A Nmax+=1

:: Poor man's "head" ..
set head=00:00:00.0
set /P head=<%tlog%
set head=%head:~15,10%
if [%head%==[ set head=00:00:00.0


:: .. and "tail".
set Mlines=0
set tail=00:00:00.0
for /F "tokens=2 usebackq" %%G in (%tlog%) do (set /A Mlines+=1&set tail=%%G)
set tail=%tail:~0,10%
if [%tail%==[ set tail=00:00:00.0


:: As proposed by David Taylor
echo.
echo Step 2/2 Extracting information from tmplog file
echo Please wait, counting Lists/Files will take time


if %Nfiles% LSS %Nmax% (echo WARNING: Nfiles ^< Nmax, test Nmax ^<= log_file_number in recv.ini>> %debug%)
if %Nfiles% EQU 0 (echo WARNING: *NO* %log_file% file found, check logpath and/or log_name >> %debug%)
if %Mlines% EQU 0 (echo WARNING: *NO* %mydate% lines in %log_file% file^(s^), ^(outdated?!^) >> %debug%)


:: Naming of output summary files. Don't change!
set out="%out%\TClogSummary_%comp%_BAS_%mydate%.txt"

:: Write a header for today's TClog summary giving due credit
echo %delim% > %out%
echo TClogSummary              Version %ver% >> %out%
echo. >> %out%
echo Windows EUMETCast receiver %comp%  (W7 x64)>> %out%
echo Shuttle DS81D i5-4570S Haswell @ 2.9GHz + 8GB >> %out%
echo Ayecka SR1 receiver Hardware version 2.00 >> %out%
echo Software version 1.05b232 , Firmware 2.02b922  >> %out%
echo Windows TelliCast Client Version %client% >> %out%
echo Eutelsat 10A DVB-S2 Basic Service >> %out%
echo. >> %out%
echo TelliCast Client Event Summary from  %mydate% >> %out%
echo Head time %head% -UTC- Tail time %tail% >> %out%
echo. >> %out%
echo A CMD script by                  Ernst Lobsiger >> %out%
echo With ideas from                  Arne van Belle >> %out%
echo %delim% >> %out%


:: Find number of ERR messages and print these messages
set sum=0
for /F %%G in ('findstr ^ERR: %tlog%') do set /A sum+=1
echo Number of messages of category ERR: %sum% >> %out%
findstr ^ERR: %tlog% >> %out%
echo %delim% >> %out%


:: Find number of WRN messages and print these messages
set sum=0
for /F %%G in ('findstr ^WRN: %tlog%') do set /A sum+=1
echo Number of messages of category WRN: %sum% >> %out%
 findstr ^WRN: %tlog% >> %out%
echo %delim% >> %out%


:: Find number of MSG messages in %tlog% but *don't* print these (-> find.exe used)
set Mlines=0
for /F "tokens=3 delims=:" %%G in ('find /I /C "MSG:" %tlog%') do (set /A Mlines=%%G)
echo Number of messages of category MSG: %Mlines% >> %out%
echo %delim% >> %out%


:: Count the number of *(re)Starting* messages and print these
set sum=0
for /F %%G in ('findstr "tarting restart" %tlog%') do set /A sum+=1
echo Number of *Starting*/*restart (what) events: %sum% >> %out%
findstr "tarting restart" %tlog% >> %out%
echo %delim% >> %out%


set message=Number of *Missed* (file) messages
:: Count the number of *Missed* file(s) and print these
set sum=0
for /F %%G in ('findstr Missed %tlog%') do set /A sum+=1
echo Number of *Missed* (file) messages: %sum% >> %out%
findstr Missed %tlog% >> %out%
echo %delim% >> %out%

set message=Number of *logging of filenames aborted* messages
:: Count the number of *aborted* line(s) and print these
set sum=0
for /F %%G in ('findstr aborted %tlog%') do set /A sum+=1
echo Number of *aborted* messages: %sum% >> %out%
findstr aborted %tlog% >> %out%
echo %delim% >> %out%

:: This is the solution as discussed with David Taylor 2011-01-17:
:: Count the number of *lost* (messages lost) / (channel lost),
:: (channel was lost), (Coordinator lost), Deconnect + Reconnect
set mess=0
for /F %%G in ('findstr "e.lost)" %tlog%') do set /A mess+=1
set chan=0
for /F %%G in ('findstr "[lsr].lost)" %tlog%') do set /A chan+=1
:: As EUMETCast works now, we output (channel lost), ..., events only
echo Number of *lost* messages/channels: %mess%/%chan% >> %out%
  findstr "[lsr].lost)" %tlog% >> %out%
:: Maybe later, with less (message lost) events, we output these too:
:: For this you may uncomment the next line and comment the line above
:: findstr ".lost)" %tlog% >> %out%
echo %delim% >> %out%


:: Ended events (are rare) lead to misssing lists or files not sent
:: Ended events must treat 3 different cases. We have to regex for:
:: "ended: filelist " "ended: 1 file " "ended: 3 files " 

:: Count the number of lines with *ended* lists or files
set lists=0
for /F %%G in ('findstr ended: %tlog%') do set /A lists+=1
:: Count ended files that have been missing/(incomplete?)
set files=0
for /F "tokens=6" %%G in ('findstr ended:.[1-9][0-9]*.file %tlog%') do set /A files+=%%G
echo Lower priority *ended* Lists/Files: %lists%/%files% >> %out%
echo %delim% >> %out%


:: Interrupted events lead to misssing/incomplete lists or files sent later
:: Interrupted events must treat 3 different cases. We have to regex for:
:: "interrupted: filelist " "interrupted: 1 file " "interrupted: 3 files " 

:: Count the number of lines with *interrupted* lists or files
set lists=0
for /F %%G in ('findstr interrupted: %tlog%') do set /A lists+=1
:: Count interrupted files that have been missing/incomplete
set files=0
for /F "tokens=6" %%G in ('findstr interrupted:.[1-9][0-9]*.file %tlog%') do set /A files+=%%G
echo Priority *interrupted* Lists/Files: %lists%/%files% >> %out%
echo %delim% >> %out%


:: Find received and delivered Lists and data Files
set lists=0
set files=0
for /F "tokens=3" %%G in ('findstr received %tlog%') do (
set /A lists+=1
set /A files+=%%G)
echo Received and delivered Lists/Files: %lists%/%files% >> %out%
echo %delim% >> %out%


:: Issue some more propaganda for this Windows CMD script ... 
echo Treated number of recv Files/Lines: %Nfiles%/%Nlines% >> %out%
echo %delim% >> %out%


:: Delete tmp files (may not exist)
if exist %tlog% del /F %tlog% 2>> %debug%
if exist %tdat% del /F %tdat% 2>> %debug%


:: V0.6+ Maybe delete older TClogSummary_*.txt files. This only works, if you have not changed file naming above!
(if %Smax% GTR 0 for /F "skip=%Smax%" %%G in ('dir /A-D /B /O-D "%org%\TClogSummary_*.txt"') do del /F "%org%\%%G") 2>>%debug%


if [%disp%==[Y cls&type %out%& pause 

:: Close that black window?
if [%called%==[STARTED exit

cls
:: Thats all folks
goto :eof



:: THE FOLLOWING CODE IS ONLY TWO LINES IN THE GNU/LINUX BASH VERSION !


:get_dates

:: This snipplet formats the "nightmare" output of 'date /T' as YYYY-MM-DD
:: Should work on any NT/2K/XP/7 machine independent of regional settings
 
SETLOCAL

:: Some code above is from http://www.ss64.com, the rest is all mine !
:: OK, I wanted to avoid that, but now we have to query the *REGISTRY* 


:: This puts 'date /T' in %%G %%H %%I %%J (%%G may contain a leading day name like 'Zo')
for /F "tokens=1-4 delims=/-. " %%G in ('date /T') do call :s_fixdate %%G %%H %%I %%J %1

:: Jump over the s_fixdate2 subroutine    
goto :FOO

:s_fixdate
   
:: If we have a leading day name we left shift parameters (%1=%2, %2=%3, %3=%4, %4=%5 %5=)
if "%1:~0,1%" GTR "9" shift

if [%4==[0 (set mm=%1&set dd=%2&set yy=%3)
if [%4==[1 (set dd=%1&set mm=%2&set yy=%3)
if [%4==[2 (set yy=%1&set mm=%2&set dd=%3)

:: Return to 'for /F ..' and jump over
goto :eof

:FOO

:: Fix cases with two digit years and/or no leading zeros for month or day
:: According to Microsoft a couple of European countries are among those !
if [%yy:~2,2%==[ set yy=20%yy%
if [%mm:~1,1%==[ set mm=0%mm%
if [%dd:~1,1%==[ set dd=0%dd%


:: Format used in recv.log  
set today=%yy%-%mm%-%dd%
set today=%today:~0,10%


:: Find yesterday from today (this method should work from 1900-03-02 to 2100-02-28)
:: The code expects a valid yy, mm, dd for today. Remember that yy is 4 digits!   EL

:: Strip leading zeros for calculations * %% -
if [%mm:~0,1%==[0 set mm=%mm:~1,1%
if [%dd:~0,1%==[0 set dd=%dd:~1,1%
set year="31 28 31 30 31 30 31 31 30 31 30 31"
set leap="31 29 31 30 31 30 31 31 30 31 30 31"
set /A mmdd=mm*dd
set /A mod4=yy%%4
set /A lastm=mm-1
if %mod4% EQU 0 set year=%leap%

:: If using &:  A 'set a=3 & ..' will lead to a='3 ' (3+blank)
if %mmdd% EQU 1 set /A yy-=1& set mm=12& set dd=31& goto :DONE
if %dd% EQU 1 set /A mm-=1& for /F "tokens=%lastm%" %%G in (%year%) do set dd=%%G& goto :DONE
set /A dd-=1
:DONE

:: Add leading zeros again
if [%mm:~1,1%==[ set mm=0%mm%
if [%dd:~1,1%==[ set dd=0%dd%

:: Format used in recv.log  
set yesterday=%yy%-%mm%-%dd%
set yesterday=%yesterday:~0,10%


ENDLOCAL&set today=%today%&set yesterday=%yesterday%
goto :eof



:: ERROR MESSAGES GO INTO A DEBUG FILE (Version 0.8+)

:start_debug

echo Debugging output of TClogSummary.cmd V %Ver% > %debug%
echo ******************************************* >> %debug%
echo. >> %debug%
echo A 'date /T' shows >> %debug%
echo ***************** >> %debug%
date /T >> %debug%
echo. >> %debug%
echo A 'time /T' shows >> %debug%
echo ***************** >> %debug%
time /T >> %debug%
echo. >> %debug%
echo How I was called >> %debug% 
echo **************** >> %debug%
echo COMPUTERNAME="%COMPUTERNAME%" >> %debug%
echo USERNAME="%USERNAME%" >> %debug%
echo CMDCMDLINE="%CMDCMDLINE%" >> %debug%
echo called=%called% >> %debug%
echo. >> %debug%
echo CLI Parameters >> %debug% 
echo ************** >> %debug% 
echo 1=%1 >> %debug% 
echo 2=%2 >> %debug% 
echo. >> %debug% 
echo Script Parameters >> %debug%
echo ***************** >> %debug%
echo comp=%comp% >> %debug%
echo client=%client% >> %debug%
echo log_file=%log_file% >> %debug%
echo logpath=%logpath% >> %debug%
echo tmppath=%tmppath% >> %debug%
echo outpath=%outpath% >> %debug%
echo Nmax=%Nmax% >> %debug%
echo Smax=%Smax% >> %debug%
echo disp=%disp% >> %debug%
echo debu=%debu% >> %debug%
echo sDate=%sDate% >> %debug%
echo delim=%delim% >> %debug%
echo srced=%srced% >> %debug%
echo. >> %debug%
echo Dates hard to find >> %debug%
echo ****************** >> %debug%
echo iDate=%iDate% >> %debug%
echo today=%today% >> %debug%
echo yesterday=%yesterday% >> %debug%
echo mydate=%mydate% >> %debug%
echo. >> %debug%
echo Expanded pathes >> %debug%
echo *************** >> %debug%
echo log=%log% >> %debug%
echo out=%out% >> %debug%
echo. >> %debug%
echo Messages >> %debug%
echo ******** >> %debug%
goto :eof
